<?php

namespace EventPortal\Domain\Form;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Arr;

class Payload
{
    /** @var array */
    private $input = [];

    /** @var Model */
    private $model;

    /** @var array<Model> */
    private $context = [];

    public function __construct(array $input)
    {
        $this->input = $input;
    }

    public static function make(array $input = []): self
    {
        return new self($input);
    }

    public function withModel(Model $model): self
    {
        $this->model = $model;

        return $this;
    }

    public function withContext(array $models = []): self
    {
        $this->context = $models;

        return $this;
    }

    public function replaceModel(Model $model): self
    {
        $this->model = $model;

        return $this;
    }

    /**
     * @param mixed $value
     */
    public function addContext(string $key, $value): self
    {
        $this->context[$key] = $value;

        return $this;
    }

    public function addToInput(array $input): self
    {
        $this->input = array_merge($this->input, $input);

        return $this;
    }

    /**
     * @return mixed
     */
    public function getContext(string $key)
    {
        return Arr::get($this->context, $key);
    }

    /**
     * @return mixed
     */
    public function getInput(?string $key = null)
    {
        if ($key) {
            return Arr::get($this->input, $key);
        }

        return $this->input;
    }

    public function getModel(): Model
    {
        return $this->model;
    }
}
